@@ -15,23 +15,31 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.02';
+$VERSION = '1.07';
 
 # BMP chunks
 %Image::ExifTool::BMP::Main = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 0 => 'File', 1 => 'File', 2 => 'Image' },
     NOTES => q{
         There really isn't much meta information in a BMP file as such, just a bit
         of image related information.
     },
+    # 0 => size of bitmap structure:
+    #        12  bytes => 'OS/2 V1',
+    #        40  bytes => 'Windows V3',
+    #        64  bytes => 'OS/2 V2',
+    #        68  bytes => some bitmap structure in AVI videos
+    #        108 bytes => 'Windows V4',
+    #        124 bytes => 'Windows V5',
     4 => {
         Name => 'ImageWidth',
         Format => 'int32u',
     },
     8 => {
         Name => 'ImageHeight',
-        Format => 'int32u',
+        Format => 'int32s', # (negative when stored in top-to-bottom order)
+        ValueConv => 'abs($val)',
     },
     12 => {
         Name => 'Planes',
@@ -53,6 +61,13 @@ $VERSION = '1.02';
             3 => 'Bitfields',
             4 => 'JPEG', #2
             5 => 'PNG', #2
+            # pass through ASCII video compression codec ID's
+            OTHER => sub {
+                my $val = shift;
+                # convert non-ascii characters
+                $val =~ s/([\0-\x1f\x7f-\xff])/sprintf('\\x%.2x',ord $1)/eg;
+                return $val;
+            },
         },
     },
     20 => {
@@ -79,6 +94,18 @@ $VERSION = '1.02';
     },
 );
 
+# OS/2 12-byte bitmap header (ref http://www.fileformat.info/format/bmp/egff.htm)
+%Image::ExifTool::BMP::OS2 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'File', 1 => 'File', 2 => 'Image' },
+    NOTES => 'Information extracted from OS/2-format BMP images.',
+    # 0 => size of bitmap structure (12)
+    4  => { Name => 'ImageWidth',  Format => 'int16u' },
+    6  => { Name => 'ImageHeight', Format => 'int16u' },
+    8  => { Name => 'Planes',      Format => 'int16u' },
+    10 => { Name => 'BitDepth',    Format => 'int16u' },
+);
+
 #------------------------------------------------------------------------------
 # Extract EXIF information from a BMP image
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -87,21 +114,26 @@ sub ProcessBMP($$)
 {
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my $buff;
+    my ($buff, $tagTablePtr);
 
     # verify this is a valid BMP file
-    return 0 unless $raf->Read($buff, 14) == 14;
+    return 0 unless $raf->Read($buff, 18) == 18;
     return 0 unless $buff =~ /^BM/;
-    return 0 unless $raf->Read($buff, 40) == 40;
     SetByteOrder('II');
-    return 0 unless Get32u(\$buff, 0) >= 40;    # make sure info is at least 40 bytes
+    my $len = Get32u(\$buff, 14);
+    return 0 unless $len == 12 or $len >= 40;
+    return 0 unless $raf->Seek(-4, 1) and $raf->Read($buff, $len) == $len;
     $exifTool->SetFileType();   # set the FileType tag
     my %dirInfo = (
         DataPt => \$buff,
         DirStart => 0,
         DirLen => length($buff),
     );
-    my $tagTablePtr = GetTagTable('Image::ExifTool::BMP::Main');
+    if ($len == 12) {   # old OS/2 format BMP
+        $tagTablePtr = GetTagTable('Image::ExifTool::BMP::OS2');
+    } else {
+        $tagTablePtr = GetTagTable('Image::ExifTool::BMP::Main');
+    }
     $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
     return 1;
 }
@@ -125,7 +157,7 @@ This module contains definitions required by Image::ExifTool to read BMP
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
