@@ -12,7 +12,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.32';
+$VERSION = '1.35';
 
 my %coordConv = (
     ValueConv    => 'Image::ExifTool::GPS::ToDegrees($val)',
@@ -40,6 +40,15 @@ my %coordConv = (
         Writable => 'string',
         Count => 2,
         PrintConv => {
+            # extract N/S if written from Composite:GPSLatitude
+            # (also allow writing from a signed number)
+            OTHER => sub {
+                my ($val, $inv) = @_;
+                return undef unless $inv;
+                return uc $1 if $val =~ /\b([NS])$/i;
+                return $1 eq '-' ? 'S' : 'N' if $val =~ /^([-+]?)\d+(\.\d*)?$/;
+                return undef;
+            },
             N => 'North',
             S => 'South',
         },
@@ -55,6 +64,15 @@ my %coordConv = (
         Writable => 'string',
         Count => 2,
         PrintConv => {
+            # extract E/W if written from Composite:GPSLongitude
+            # (also allow writing from a signed number)
+            OTHER => sub {
+                my ($val, $inv) = @_;
+                return undef unless $inv;
+                return uc $1 if $val =~ /\b([EW])$/i;
+                return $1 eq '-' ? 'W' : 'E' if $val =~ /^([-+]?)\d+(\.\d*)?$/;
+                return undef;
+            },
             E => 'East',
             W => 'West',
         },
@@ -248,6 +266,7 @@ my %coordConv = (
     0x001b => {
         Name => 'GPSProcessingMethod',
         Writable => 'undef',
+        Notes => 'values of "GPS", "CELLID", "WLAN" or "MANUAL" by the EXIF spec.',
         RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
         RawConvInv => 'Image::ExifTool::Exif::EncodeExifText($self,$val)',
     },
@@ -261,7 +280,8 @@ my %coordConv = (
         Name => 'GPSDateStamp',
         Groups => { 2 => 'Time' },
         Writable => 'string',
-        Notes => 'YYYY:MM:DD',
+        Format => 'undef', # (Casio EX-H20G uses "\0" instead of ":" as a separator)
+        Notes => 'YYYY:mm:dd',
         Count => 11,
         Shift => 'Time',
         Notes => q{
@@ -288,6 +308,13 @@ my %coordConv = (
             1 => 'Differential Corrected',
         },
     },
+    0x001f => {
+        Name => 'GPSHPositioningError',
+        Description => 'GPS Horizontal Positioning Error',
+        PrintConv => '"$val m"',
+        PrintConvInv => '$val=~s/\s*m$//; $val',
+        Writable => 'rational64u',
+    },
 );
 
 # Composite GPS tags
@@ -326,18 +353,19 @@ my %coordConv = (
     },
     GPSAltitude => {
         SubDoc => 1,    # generate for all sub-documents
-        Require => {
-            0 => 'GPSAltitudeRef',
-        },
         Desire => {
-            1 => 'GPS:GPSAltitude',
+            0 => 'GPS:GPSAltitude',
+            1 => 'GPS:GPSAltitudeRef',
             2 => 'XMP:GPSAltitude',
+            3 => 'XMP:GPSAltitudeRef',
         },
+        # Require either GPS:GPSAltitudeRef or XMP:GPSAltitudeRef
+        RawConv => '(defined $val[1] or defined $val[3]) ? $val : undef',
         ValueConv => q{
-            my $alt = $val[1];
+            my $alt = $val[0];
             $alt = $val[2] unless defined $alt;
             return undef unless defined $alt;
-            return $val[0] ? -$alt : $alt;
+            return ($val[1] || $val[3]) ? -$alt : $alt;
         },
         PrintConv => q{
             $val = int($val * 10) / 10;
@@ -451,7 +479,7 @@ GPS (Global Positioning System) meta information in EXIF data.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
