@@ -18,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:Public);
 
-$VERSION = '1.16';
+$VERSION = '1.24';
 
 sub SetGeoValues($$;$);
 
@@ -46,6 +46,7 @@ my %xmlTag = (
     # XML containers (fix is reset at the opening tag of these properties)
     wpt         => '',          # GPX
     trkpt       => '',          # GPX
+    rtept       => '',          # GPX
     trackpoint  => '',          # Garmin
     placemark   => '',          # KML
 );
@@ -87,23 +88,39 @@ sub LoadTrackLog($$;$)
     }
     # add data to existing track
     my $geotag = $exifTool->GetNewValues('Geotag') || { };
-    if ($val =~ /(\x0d\x0a|\x0d|\x0a)/) {
-        # $val is track log data
+    my $format = '';
+    # is $val track log data?
+    if ($val =~ /^(\xef\xbb\xbf)?<(\?xml|gpx)\s/) {
+        $format = 'XML';
+        $/ = '>';   # set input record separator to '>' for XML/GPX data
+    } elsif ($val =~ /(\x0d\x0a|\x0d|\x0a)/) {
         $/ = $1;
-        $raf = new File::RandomAccess(\$val);
-        $from = 'data';
     } else {
         # $val is track file name
         open EXIFTOOL_TRKFILE, $val or return "Error opening GPS file '$val'";
         $raf = new File::RandomAccess(\*EXIFTOOL_TRKFILE);
-        unless ($raf->Read($_, 256) and /(\x0d\x0a|\x0d|\x0a)/) {
+        unless ($raf->Read($_, 256)) {
+            close EXIFTOOL_TRKFILE;
+            return "Empty track file '$val'";
+        }
+        # look for XML or GPX header (might as well allow UTF-8 BOM)
+        if (/^(\xef\xbb\xbf)?<(\?xml|gpx)\s/) {
+            $format = 'XML';
+            $/ = '>';   # set input record separator to '>' for XML/GPX data
+        } elsif (/(\x0d\x0a|\x0d|\x0a)/) {
+            $/ = $1;
+        } else {
             close EXIFTOOL_TRKFILE;
             return "Invalid track file '$val'";
         }
-        $/ = $1;
         $raf->Seek(0,0);
         $from = "file '$val'";
     }
+    unless ($from) {
+        # set up RAF for reading log file in memory
+        $raf = new File::RandomAccess(\$val);
+        $from = 'data';
+    }
     # initialize track points lookup
     my $points = $$geotag{Points};
     $points or $points = $$geotag{Points} = { };
@@ -117,7 +134,6 @@ sub LoadTrackLog($$;$)
     my $numPoints = 0;
     my $skipped = 0;
     my $lastSecs = 0;
-    my $format = '';
     my $fix = { };
     for (;;) {
         $raf->ReadLine($_) or last;
@@ -182,7 +198,7 @@ sub LoadTrackLog($$;$)
                         if ($tag) {
                             if ($tag eq 'coords') {
                                 # read KML "Point" coordinates
-                                @$fix{'lat','lon','alt'} = split ',', $1;
+                                @$fix{'lon','lat','alt'} = split ',', $1;
                             } else {
                                 $$fix{$tag} = $1;
                             }
@@ -192,7 +208,7 @@ sub LoadTrackLog($$;$)
                         $td = 1;
                     }
                     # validate and store GPS fix
-                    if ($$fix{lat} and $$fix{lon} and $$fix{'time'} and
+                    if (defined $$fix{lat} and defined $$fix{lon} and $$fix{'time'} and
                         $$fix{lat} =~ /^[+-]?\d+\.?\d*/ and
                         $$fix{lon} =~ /^[+-]?\d+\.?\d*/ and
                         $$fix{'time'} =~ /^(\d{4})-(\d+)-(\d+)T(\d+):(\d+):(\d+)(\.\d+)?(.*)/)
@@ -201,13 +217,15 @@ sub LoadTrackLog($$;$)
                         $time += $7 if $7;  # add fractional seconds
                         my $tz = $8;
                         # adjust for time zone (otherwise assume UTC)
-                        if ($tz =~ /^([-+])(\d+):(\d{2})\b/) {
-                            $tz = ($2 * 60 + $3) * 60;
+                        # - allow timezone of +-HH:MM, +-H:MM, +-HHMM or +-HH since
+                        #   the spec is unclear about timezone format
+                        if ($tz =~ /^([-+])(\d+):(\d{2})\b/ or $tz =~ /^([-+])(\d{2})(\d{2})?\b/) {
+                            $tz = ($2 * 60 + ($3 || 0)) * 60;
                             $tz *= -1 if $1 eq '+'; # opposite sign to change back to UTC
                             $time += $tz;
                         }
                         # validate altitude
-                        undef $$fix{alt} if $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
+                        undef $$fix{alt} if defined $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
                         $isDate = 1;
                         $canCut= 1 if defined $$fix{pdop} or defined $$fix{hdop} or defined $$fix{nsats};
                         $$points{$time} = $fix;
@@ -501,7 +519,7 @@ sub ApplySyncCorr($$)
             my ($t0, $t1) = ($$syncTimes[$i0], $$syncTimes[$i1]);
             # interpolate/extrapolate to account for linear camera clock drift
             my $syncPoints = $$sync{Points};
-            my $f = ($time - $t0) / ($t1 - $t0);
+            my $f = $t1 == $t0 ? 0 : ($time - $t0) / ($t1 - $t0);
             $sync = $$syncPoints{$t1} * $f + $$syncPoints{$t0} * (1 - $f);
         } else {
             $sync = $$sync{Offset}; # use fixed time offset
@@ -570,7 +588,7 @@ sub SetGeoValues($$;$)
             ($year,$mon,$day) = (1970,1,2);
             $noDate = 1;
         } else {
-            $err = 'Invalid date/time (use YYYY:MM:DD HH:MM:SS[.SS][+/-HH:MM|Z])';
+            $err = 'Invalid date/time (use YYYY:mm:dd HH:MM:SS[.ss][+/-HH:MM|Z])';
             last;
         }
         if ($tz) {
@@ -642,7 +660,7 @@ sub SetGeoValues($$;$)
                     $fix = $$points{$tn};
                 }
             } else {
-                my $f = ($time - $t0) / ($t1 - $t0);
+                my $f = $t1 == $t0 ? 0 : ($time - $t0) / ($t1 - $t0);
                 my $p0 = $$points{$t0};
                 $fix = { };
                 # loop through latitude, longitude, and altitude if available
@@ -668,9 +686,9 @@ sub SetGeoValues($$;$)
         # write GPSDateStamp if date included in track log, otherwise delete it
         $gpsDate = sprintf('%.2d:%.2d:%.2d', $t[5]+1900, $t[4]+1, $t[3]) unless $noDate;
         # write GPSAltitude tags if altitude included in track log, otherwise delete them
-        if ($$fix{alt}) {
+        if (defined $$fix{alt}) {
             $gpsAlt = abs $$fix{alt};
-            $gpsAltRef = ($$fix{alt} > 0 ? 0 : 1);
+            $gpsAltRef = ($$fix{alt} < 0 ? 1 : 0);
         }
         # set new GPS tag values (EXIF, or XMP if write group is 'xmp')
         my ($xmp, $exif, @r);
@@ -725,14 +743,15 @@ sub ConvertGeosync($$)
     my $sync = $exifTool->GetNewValues('Geosync') || { };
     my ($syncFile, $gpsTime, $imgTime);
 
-    if ($val =~ /(.*?)@(.*)/) {
+    if ($val =~ /(.*?)\@(.*)/) {
         $gpsTime = $1;
         if (-f $2) {
             $syncFile = $2;
         } else {
             $imgTime = $2;
         }
-    } else {
+    # (take care because "-f '1:30'" crashes ActivePerl 5.10)
+    } elsif ($val !~ /^\d/ or $val !~ /:/) {
         $syncFile = $val if -f $val;
     }
     if ($gpsTime or defined $syncFile) {
@@ -863,7 +882,7 @@ in the tag name documentation).
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
