@@ -14,7 +14,7 @@ package Image::ExifTool::IPTC;
 use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 
-$VERSION = '1.36';
+$VERSION = '1.42';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -109,6 +109,12 @@ my %fileFormat = (
             TagTable => 'Image::ExifTool::IPTC::PostObjectData',
         },
     },
+    240 => {
+        Name => 'IPTCFotoStation',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::IPTC::FotoStation',
+        },
+    },
 );
 
 # Record 1 -- EnvelopeRecord
@@ -488,9 +494,9 @@ my %fileFormat = (
         Name => 'Caption-Abstract',
         Format => 'string[0,2000]',
     },
-    121 => { # (format not certain)
+    121 => {
         Name => 'LocalCaption',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
         Notes => q{
             I haven't found a reference for the format of tags 121, 184-188 and
             225-232, so I have just make them writable as strings with
@@ -506,7 +512,7 @@ my %fileFormat = (
     },
     125 => {
         Name => 'RasterizedCaption',
-        Format => 'string[7360]',
+        Format => 'undef[7360]',
         Binary => 1,
     },
     130 => {
@@ -567,25 +573,25 @@ my %fileFormat = (
         Name => 'AudioOutcue',
         Format => 'string[0,64]',
     },
-    184 => { # (format not certain)
+    184 => {
         Name => 'JobID',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    185 => { # (format not certain)
+    185 => {
         Name => 'MasterDocumentID',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
     },
-    186 => { # (format not certain)
+    186 => {
         Name => 'ShortDocumentID',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    187 => { # (format not certain)
+    187 => {
         Name => 'UniqueDocumentID',
-        Format => 'string[0,128]',
+        Format => 'string[0,128]', # (guess)
     },
-    188 => { # (format not certain)
+    188 => {
         Name => 'OwnerID',
-        Format => 'string[0,128]',
+        Format => 'string[0,128]', # (guess)
     },
     200 => {
         Name => 'ObjectPreviewFileFormat',
@@ -601,7 +607,7 @@ my %fileFormat = (
     202 => {
         Name => 'ObjectPreviewData',
         Groups => { 2 => 'Image' },
-        Format => 'string[0,256000]',
+        Format => 'undef[0,256000]',
         Binary => 1,
     },
     221 => {
@@ -620,25 +626,31 @@ my %fileFormat = (
             return $val;
         },
     },
-    225 => { # (format not certain)
+    225 => {
         Name => 'ClassifyState',
-        Format => 'string[0,64]',
+        Format => 'string[0,64]', # (guess)
     },
-    228 => { # (format not certain)
+    228 => {
         Name => 'SimilarityIndex',
-        Format => 'string[0,32]',
+        Format => 'string[0,32]', # (guess)
     },
-    230 => { # (format not certain)
+    230 => {
         Name => 'DocumentNotes',
-        Format => 'string[0,1024]',
+        Format => 'string[0,1024]', # (guess)
     },
-    231 => { # (format not certain)
+    231 => {
         Name => 'DocumentHistory',
-        Format => 'string[0,256]',
+        Format => 'string[0,256]', # (guess)
     },
-    232 => { # (format not certain)
+    232 => {
         Name => 'ExifCameraInfo',
-        Format => 'string[0,4096]',
+        Format => 'string[0,4096]', # (guess)
+    },
+    255 => { #PH
+        Name => 'CatalogSets',
+        List => 1,
+        Format => 'string[0,256]', # (guess)
+        Notes => 'written by iView MediaPro',
     },
 );
 
@@ -877,6 +889,14 @@ my %fileFormat = (
     },
 );
 
+# Record 240 -- FotoStation proprietary data (ref PH)
+%Image::ExifTool::IPTC::FotoStation = (
+    GROUPS => { 2 => 'Other' },
+    WRITE_PROC => \&WriteIPTC,
+    CHECK_PROC => \&CheckIPTC,
+    WRITABLE => 1,
+);
+
 # IPTC Composite tags
 %Image::ExifTool::IPTC::Composite = (
     GROUPS => { 2 => 'Image' },
@@ -956,7 +976,6 @@ sub HandleCodedCharset($$)
 sub TranslateCodedString($$$$)
 {
     my ($exifTool, $valPtr, $xlatPtr, $read) = @_;
-    my $escaped;
     if ($$xlatPtr eq 'bad') {
         $exifTool->Warn('Some IPTC characters not converted (unsupported CodedCharacterSet)');
         undef $$xlatPtr;
@@ -964,13 +983,22 @@ sub TranslateCodedString($$$$)
         $$valPtr = $exifTool->Decode($$valPtr, undef, undef, $$xlatPtr);
     } elsif ($$valPtr !~ /[\x14\x15\x1b]/) {
         $$valPtr = $exifTool->Decode($$valPtr, $$xlatPtr);
-    } elsif (not $$exifTool{WarnShift2022}) {
+    } else {
         # don't yet support reading ISO 2022 shifted character sets
-        $exifTool->Warn('Some IPTC characters not converted (ISO 2022 shifting not supported)');
-        $$exifTool{WarnShift2022} = 1;
+        $exifTool->WarnOnce('Some IPTC characters not converted (ISO 2022 shifting not supported)');
     }
 }
 
+#------------------------------------------------------------------------------
+# Is this IPTC in a standard location?
+# Inputs: 0) Current metadata path string
+# Returns: true if path is standard
+sub IsStandardIPTC($)
+{
+    my $path = shift;
+    return $path =~ /^(JPEG-APP13-Photoshop-IPTC|TIFF-IFD0-IPTC|PSD-IPTC|MIE-IPTC)$/
+}
+
 #------------------------------------------------------------------------------
 # get IPTC info
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -991,8 +1019,8 @@ sub ProcessIPTC($$$)
 
     if ($tagTablePtr eq \%Image::ExifTool::IPTC::Main) {
         # calculate MD5 if Digest::MD5 is available (for standard IPTC only)
-        my $path = join('-', @{$$exifTool{PATH}});
-        if ($path =~ /^(JPEG-APP13-Photoshop-IPTC|TIFF-IFD0-IPTC|PSD-IPTC|MIE-IPTC)$/) {
+        my $path = $exifTool->MetadataPath();
+        if (IsStandardIPTC($path)) {
             my $md5;
             if (eval 'require Digest::MD5') {
                 if ($pos or $dirLen != length($$dataPt)) {
@@ -1117,7 +1145,7 @@ sub ProcessIPTC($$$)
                 }
             } elsif ($format =~ /^digits/) {
                 $val =~ s/\0+$//;
-            } else {
+            } elsif ($format !~ /^undef/) {
                 warn("Invalid IPTC format: $format");
             }
         }
@@ -1129,6 +1157,7 @@ sub ProcessIPTC($$$)
             Size    => $len,
             Start   => $pos,
             Extra   => ", $recordName record",
+            Format  => $format,
         );
         $exifTool->FoundTag($tagInfo, $val) if $tagInfo;
         $success = 1;
@@ -1160,7 +1189,7 @@ image files.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
