@@ -15,12 +15,14 @@ use Image::ExifTool::Exif;
 
 sub ProcessUnknown($$$);
 sub ProcessUnknownOrPreview($$$);
+sub ProcessCanon($$$);
+sub ProcessGE2($$$);
 sub WriteUnknownOrPreview($$$);
 sub FixLeicaBase($$;$);
 
-$VERSION = '1.56';
+$VERSION = '1.67';
 
-my $debug;          # set to 1 to enabled debugging code
+my $debug;          # set to 1 to enable debugging code
 
 # conditional list of maker notes
 # Notes:
@@ -36,6 +38,7 @@ my $debug;          # set to 1 to enabled debugging code
         Condition => '$$self{Make} =~ /^Canon/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Canon::Main',
+            ProcessProc => \&ProcessCanon,
             ByteOrder => 'Unknown',
         },
     },
@@ -44,7 +47,7 @@ my $debug;          # set to 1 to enabled debugging code
         # do negative lookahead assertion just to get tags
         # in a nice order for documentation
         # (starts with an IFD)
-        Condition => '$$self{Make}=~/^CASIO(?! COMPUTER CO.,LTD)/',
+        Condition => '$$self{Make}=~/^CASIO/ and $$valPt!~/^(QVC|DCI)\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Main',
             ByteOrder => 'Unknown',
@@ -52,9 +55,9 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteCasio2',
-        # (starts with "QVC\0")
+        # (starts with "QVC\0" [Casio] or "DCI\0" [Concord])
         # (also found in AVI and MOV videos)
-        Condition => '$$self{Make}=~/^CASIO COMPUTER CO.,LTD/',
+        Condition => '$$valPt =~ /^(QVC|DCI)\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Type2',
             Start => '$valuePtr + 6',
@@ -67,7 +70,8 @@ my $debug;          # set to 1 to enabled debugging code
         # TIFF file, but with "FUJIFILM" instead of the standard TIFF header
         Name => 'MakerNoteFujiFilm',
         # (starts with "FUJIFILM" -- also used by some Leica, Minolta and Sharp models)
-        Condition => '$$valPt =~ /^FUJIFILM/',
+        # (GE FujiFilm models start with "GENERALE")
+        Condition => '$$valPt =~ /^(FUJIFILM|GENERALE)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::FujiFilm::Main',
             # there is an 8-byte maker tag (FUJIFILM) we must skip over
@@ -78,6 +82,34 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'LittleEndian',
         },
     },
+    {
+        Name => 'MakerNoteGE',
+        Condition => '$$valPt =~ /^GE(\0\0|NIC\0)/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::GE::Main',
+            Start => '$valuePtr + 18',
+            FixBase => 1,
+            AutoFix => 1,
+            ByteOrder => 'Unknown',
+       },
+    },
+    {
+        Name => 'MakerNoteGE2',
+        Condition => '$$valPt =~ /^GE\x0c\0\0\0\x16\0\0\0/',
+        # Note: we will get a "Maker notes could not be parsed" warning when writing
+        #       these maker notes because they aren't currently supported for writing
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::FujiFilm::Main',
+            ProcessProc => \&ProcessGE2,
+            Start => '$valuePtr + 12',
+            Base => '$start - 6',
+            ByteOrder => 'LittleEndian',
+            # hard patch for crazy offsets
+            FixOffsets => '$valuePtr -= 210 if $tagID >= 0x1303',
+       },
+    },
+    # (the GE X5 has really messed up EXIF-like maker notes starting with
+    #  "GENIC\x0c\0" --> currently not decoded)
     {
         Name => 'MakerNoteHP',  # PhotoSmart 720 (also Vivitar 3705, 3705B and 3715)
         Condition => '$$valPt =~ /^(Hewlett-Packard|Vivitar)/',
@@ -183,7 +215,11 @@ my $debug;          # set to 1 to enabled debugging code
         # not much to key on here, but we know the
         # upper byte of the year should be 0x07:
         Name => 'MakerNoteKodak3',
-        Condition => '$$self{Make}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        Condition => q{
+            $$self{Make} =~ /^EASTMAN KODAK/ and
+            $$valPt =~ /^(?!MM|II).{12}\x07/s and
+            $$valPt !~ /^(MM|II|AOC)/
+        },
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type3',
@@ -192,7 +228,11 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak4',
-        Condition => '$$self{Make}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        Condition => q{
+            $$self{Make} =~ /^Eastman Kodak/ and
+            $$valPt =~ /^.{41}JPG/s and
+            $$valPt !~ /^(MM|II|AOC)/
+        },
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type4',
@@ -285,10 +325,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak9',
         # test header and Kodak:DateTimeOriginal
-        Condition => q{
-            $$self{Make}=~/Kodak/i and
-            $$valPt =~ /^IIII.{16}\d{4}\/\d{2}\/\d{2} /s
-        },
+        Condition => '$$valPt =~ m{^IIII[\x02\x03]\0.{14}\d{4}/\d{2}/\d{2} }s',
         NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type9',
@@ -323,7 +360,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKyocera',
         # (starts with "KYOCERA")
-        Condition => '$$self{Make}=~/^KYOCERA/',
+        Condition => '$$valPt =~ /^KYOCERA/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Unknown::Main',
             Start => '$valuePtr + 22',
@@ -363,7 +400,6 @@ my $debug;          # set to 1 to enabled debugging code
         Name => 'MakerNoteMinolta3',
         Condition => '$$self{Make} =~ /^(Konica Minolta|Minolta)/i',
         Binary => 1,
-        NotIFD => 1,
         Notes => 'not EXIF-based',
     },
     {
@@ -498,7 +534,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic',
         # (starts with "Panasonic\0")
-        Condition => '$$self{Make} =~ /^Panasonic/ and $$valPt!~/^MKE/',
+        Condition => '$$valPt=~/^Panasonic/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Main',
             Start => '$valuePtr + 12',
@@ -508,7 +544,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic2',
         # (starts with "Panasonic\0")
-        Condition => '$$self{Make} =~ /^Panasonic/',
+        Condition => '$$self{Make}=~/^Panasonic/ and $$valPt=~/^MKE/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Type2',
             ByteOrder => 'LittleEndian',
@@ -569,6 +605,50 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'LittleEndian',
         },
     },
+    {
+        Name => 'MakerNotePentax5',
+        # (starts with "PENTAX \0")
+        Condition => '$$valPt=~/^PENTAX \0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::Main',
+            Start => '$valuePtr + 10',
+            Base => '$start - 10',
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax6',
+        # (starts with "S1\0\0\0\0\0\0\x0c\0\0\0")
+        Condition => '$$valPt=~/^S1\0{6}\x0c\0{3}/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::S1',
+            Start => '$valuePtr + 12',
+            Base => '$start - 12',
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePhaseOne',
+        # Starts with: 'IIIITwaR' or 'IIIICwaR' (have seen both written by P25)
+        # (have also seen code which expects 'MMMMRawT')
+        Condition => q{
+            return undef unless $$valPt =~ /^(IIII.waR|MMMMRaw.)/s;
+            $self->OverrideFileType($$self{TIFF_TYPE} = 'IIQ') if $count > 1000000;
+            return 1;
+        },
+        NotIFD => 1,
+        Binary => 1,
+        PutFirst => 1, # place immediately after TIFF header
+        Notes => 'the raw image data in PhaseOne IIQ images',
+    },
+    {
+        Name => 'MakerNoteReconyx',
+        Condition => '$$valPt =~ /^\x01\xf1[\x02\x03]\x00/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Reconyx::Main',
+            ByteOrder => 'Little-endian',
+        },
+    },
     {
         Name => 'MakerNoteRicoh',
         # (my test R50 image starts with "      \x02\x01" - PH)
@@ -589,19 +669,29 @@ my $debug;          # set to 1 to enabled debugging code
         },
     },
     {
-        Name => 'MakerNoteSamsung',
-        Condition => '$$valPt =~ /^STMN\d{3}/',
+        Name => 'MakerNoteSamsung1a',
+        # Samsung STMN maker notes WITHOUT PreviewImage
+        Condition => '$$valPt =~ /^STMN\d{3}.\0{4}/s',
         Binary => 1,
-        Notes => 'Samsung unknown maker notes',
+        Notes => 'Samsung "STMN" maker notes without PreviewImage',
+    },
+    {
+        Name => 'MakerNoteSamsung1b',
+        # Samsung STMN maker notes WITH PreviewImage
+        Condition => '$$valPt =~ /^STMN\d{3}/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Samsung::Type1',
+        },
     },
     {
         Name => 'MakerNoteSamsung2',
+        # Samsung EXIF-format maker notes
         Condition => q{
             $$self{Make} eq 'SAMSUNG' and ($$self{TIFF_TYPE} eq 'SRW' or
             $$valPt=~/^(\0.\0\x01\0\x07\0{3}\x04|.\0\x01\0\x07\0\x04\0{3})0100/s)
         },
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Samsung::Main',
+            TagTable => 'Image::ExifTool::Samsung::Type2',
             # Samsung is very inconsistent here, and uses absolute offsets for some
             # models and relative offsets for others, so process as Unknown
             ProcessProc => \&ProcessUnknown,
@@ -694,6 +784,16 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteSonyEricsson',
+        Condition => '$$valPt =~ /^SEMC MS\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Sony::Ericsson',
+            Start => '$valuePtr + 20',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteSonySRF',
         Condition => '$$self{Make}=~/^SONY/',
@@ -703,6 +803,14 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteUnknownText',
+        Condition => '$$valPt =~ /^[\x09\x0d\x0a\x20-\x7e]+\0*$/',
+        Notes => 'unknown text-based maker notes',
+        # show as binary if it is too long
+        ValueConv => 'length($val) > 64 ? \$val : $val',
+        ValueConvInv => '$val',
+    },
     {
         Name => 'MakerNoteUnknown',
         PossiblePreview => 1,
@@ -723,7 +831,7 @@ foreach $tagInfo (@Image::ExifTool::MakerNotes::Main) {
     $$tagInfo{WriteGroup} = 'ExifIFD';
     $$tagInfo{Groups} = { 1 => 'MakerNotes' };
     next unless $$tagInfo{SubDirectory};
-    # set up this tag so we can write it
+    # make all SubDirectory tags block-writable
     $$tagInfo{Binary} = 1,
     $$tagInfo{MakerNotes} = 1;
 }
@@ -755,8 +863,11 @@ sub GetMakerNoteOffset($)
         push @offsets, 16 if $model =~ /(PowerShot|IXUS|IXY)/;
     } elsif ($make =~ /^CASIO/) {
         # Casio AVI and MOV images use no padding, and their JPEG's use 4,
-        # except some models like the EX-S770,Z65,Z70,Z75 and Z700 which use 16
-        push @offsets, $$exifTool{FILE_TYPE} =~ /^(RIFF|MOV)$/ ? 0 : (4, 16);
+        # except some models like the EX-S770,Z65,Z70,Z75 and Z700 which use 16,
+        # and the EX-Z35 which uses 2 (grrrr...)
+        push @offsets, $$exifTool{FILE_TYPE} =~ /^(RIFF|MOV)$/ ? 0 : (4, 16, 2);
+    } elsif ($make =~ /^(General Imaging Co.|GEDSC IMAGING CORP.)/i) {
+        push @offsets, 0;
     } elsif ($make =~ /^KYOCERA/) {
         push @offsets, 12;
     } elsif ($make =~ /^Leica Camera AG/) {
@@ -882,21 +993,13 @@ sub FixBase($$)
 # handle special case of Canon maker notes with TIFF footer containing original offset
 #
     if ($$exifTool{Make} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
-        my $trailerPos = $dirStart + $$dirInfo{DirLen} - 8;
-        my $footer = substr($$dataPt, $trailerPos, 8);
+        my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
+        my $footer = substr($$dataPt, $footerPos, 8);
         if ($footer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF footer
             substr($footer,0,2) eq GetByteOrder()) # validate byte ordering
         {
             my $oldOffset = Get32u(\$footer, 4);
             my $newOffset = $dirStart + $dataPos;
-            if ($$exifTool{HTML_DUMP}) {
-                my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $trailerPos;
-                my $str = sprintf('Original maker note offset: 0x%.4x', $oldOffset);
-                if ($oldOffset != $newOffset) {
-                    $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
-                }
-                $exifTool->HDump($filePos, 8, '[Canon MakerNotes footer]', $str);
-            }
             if ($setBase) {
                 $fix = $fixBase;
             } else {
@@ -1218,6 +1321,14 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
                 # count must be reasonable (can't test for zero count because
                 # cameras like the 1DmkIII use this value)
                 next IFD_TRY if $count & 0xff000000;
+                # extra tests to avoid mis-identifying Samsung makernotes (GT-I9000, etc)
+                next unless $num == 1;
+                my $valueSize = $count * $Image::ExifTool::Exif::formatSize[$format];
+                if ($valueSize > 4) {
+                    next IFD_TRY if $valueSize > $size;
+                    my $valuePtr = Get32u($dataPt, $entry+8);
+                    next IFD_TRY if $valuePtr > 0x10000;
+                }
             }
             $$dirInfo{DirStart} += $offset;    # update directory start
             $$dirInfo{DirLen} -= $offset;
@@ -1255,6 +1366,51 @@ sub FixLeicaBase($$;$)
     return $success;
 }
 
+#------------------------------------------------------------------------------
+# Process Canon maker notes
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessCanon($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    # identify Canon MakerNote footer in HtmlDump
+    # (this code moved from FixBase so it also works for Adobe MakN in DNG images)
+    if ($$exifTool{HTML_DUMP} and $$dirInfo{DirLen} > 8) {
+        my $dataPos = $$dirInfo{DataPos};
+        my $dirStart = $$dirInfo{DirStart} || 0;
+        my $footerPos = $dirStart + $$dirInfo{DirLen} - 8;
+        my $footer = substr(${$$dirInfo{DataPt}}, $footerPos, 8);
+        if ($footer =~ /^(II\x2a\0|MM\0\x2a)/ and substr($footer,0,2) eq GetByteOrder()) {
+            my $oldOffset = Get32u(\$footer, 4);
+            my $newOffset = $dirStart + $dataPos;
+            my $str = sprintf('Original maker note offset: 0x%.4x', $oldOffset);
+            if ($oldOffset != $newOffset) {
+                $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
+            }
+            my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $footerPos;
+            $exifTool->HDump($filePos, 8, '[Canon MakerNotes footer]', $str);
+        }
+    }
+    # process as normal
+    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+}
+
+#------------------------------------------------------------------------------
+# Process GE type 2 maker notes
+# Inputs: 0) ExifTool object ref, 1) DirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessGE2($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt} or return 0;
+    my $dirStart = $$dirInfo{DirStart} || 0;
+
+    # these maker notes are missing the IFD entry count, but they
+    # always have 25 entries, so write the entry count manually
+    Set16u(25, $dataPt, $dirStart);
+    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+}
+
 #------------------------------------------------------------------------------
 # Process unknown maker notes or PreviewImage
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -1356,7 +1512,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
