@@ -4,6 +4,8 @@
 # Description:  Read/write Nikon Capture information
 #
 # Revisions:    11/08/2005 - P. Harvey Created
+#               10/10/2008 - P. Harvey Updated for Capture NX 2
+#               16/04/2011 - P. Harvey Decode NikonCaptureEditVersions
 #
 # References:   1) http://www.cybercom.net/~dcoffin/dcraw/
 #------------------------------------------------------------------------------
@@ -15,7 +17,9 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.02';
+$VERSION = '1.07';
+
+sub ProcessNikonCapture($$$);
 
 # common print conversions
 my %offOn = ( 0 => 'Off', 1 => 'On' );
@@ -40,6 +44,7 @@ my %unsharpColor = (
         This information is written by the Nikon Capture software in tag 0x0e01 of
         the maker notes of NEF images.
     },
+    # 0x007ddc9d contains contrast information
     0x008ae85e => {
         Name => 'LCHEditor',
         Writable => 'int8u',
@@ -50,6 +55,12 @@ my %unsharpColor = (
         Writable => 'int8u',
         PrintConv => \%offOn,
     },
+    0x116fea21 => {
+        Name => 'HighlightData',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::NikonCapture::HighlightData',
+        },
+    },
     0x2175eb78 => {
         Name => 'D-LightingHQ',
         Writable => 'int8u',
@@ -65,12 +76,29 @@ my %unsharpColor = (
             TagTable => 'Image::ExifTool::NikonCapture::CropData',
         },
     },
+    0x39c456ac => {
+        Name => 'PictureCtrl',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::NikonCapture::PictureCtrl',
+        },
+    },
     0x3cfc73c6 => {
         Name => 'RedEyeData',
         SubDirectory => {
             TagTable => 'Image::ExifTool::NikonCapture::RedEyeData',
         },
     },
+    0x3d136244 => {
+        Name => 'EditVersionName',
+        Writable => 'string', # (null terminated)
+    },
+    # 0x3e726567 added when I rotated by 90 degrees
+    0x56a54260 => {
+        Name => 'Exposure',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::NikonCapture::Exposure',
+        },
+    },
     0x5f0e7d23 => {
         Name => 'ColorBooster',
         Writable => 'int8u',
@@ -125,6 +153,12 @@ my %unsharpColor = (
         Name => 'Rotation',
         Writable => 'int16u',
     },
+    0x083a1a25 => {
+        Name => 'HistogramXML',
+        Writable => 'undef',
+        Binary => 1,
+        AdjustSize => 4,    # patch Nikon bug
+    },
     0x84589434 => {
         Name => 'BrightnessData',
         SubDirectory => {
@@ -149,6 +183,7 @@ my %unsharpColor = (
             TagTable => 'Image::ExifTool::IPTC::Main',
         },
     },
+  # 0xa7264a72, 0x88f55e48 and 0x416391c6 all change from 0 to 1 when QuickFix is turned on
     0xab5eca5e => {
         Name => 'PhotoEffects',
         Writable => 'int8u',
@@ -181,6 +216,11 @@ my %unsharpColor = (
         Writable => 'int8u',
         PrintConv => \%offOn,
     },
+    0xe2173c47 => {
+        Name => 'PictureControl',
+        Writable => 'int8u',
+        PrintConv => \%offOn,
+    },
     0xe37b4337 => {
         Name => 'D-LightingHSData',
         SubDirectory => {
@@ -193,6 +233,16 @@ my %unsharpColor = (
             TagTable => 'Image::ExifTool::NikonCapture::UnsharpData',
         },
     },
+    0xe9651831 => {
+        Name => 'PhotoEffectHistoryXML',
+        Binary => 1,
+        Writable => 'undef',
+    },
+    0xfe28a44f => {
+        Name => 'AutoRedEye',
+        Writable => 'int8u',
+        PrintConv => \%offOn, # (have seen a value of 28 here for older software?)
+    },
     0xfe443a45 => {
         Name => 'ImageDustOff',
         Writable => 'int8u',
@@ -248,9 +298,9 @@ my %unsharpColor = (
     FORMAT => 'int32u',
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    0 => 'D-LightingHSShadow',
-    1 => 'D-LightingHSHighlight',
-    2 => 'D-LightingHSColorBoost',
+    0 => 'D-LightingHQShadow',
+    1 => 'D-LightingHQHighlight',
+    2 => 'D-LightingHQColorBoost',
 );
 
 %Image::ExifTool::NikonCapture::ColorBoost = (
@@ -299,6 +349,11 @@ my %unsharpColor = (
             4 => 'Calculate Automatically'
         },
     },
+    0x14 => {
+        Name => 'WBAdjLightingSubtype',
+        # this varies for different lighting types
+        # (ie. for Daylight, this is 0 => 'Direct', 1 => 'Shade', 2 => 'Cloudy')
+    },
     0x15 => {
         Name => 'WBAdjLighting',
         PrintConv => {
@@ -314,6 +369,10 @@ my %unsharpColor = (
         Name => 'WBAdjTemperature',
         Format => 'int16u',
     },
+    0x25 => {
+        Name => 'WBAdjTint',
+        Format => 'int32s',
+    },
 );
 
 %Image::ExifTool::NikonCapture::PhotoEffects = (
@@ -375,12 +434,12 @@ my %unsharpColor = (
     FORMAT => 'int8u',
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    4 => {
+    0x04 => {
         Name => 'EdgeNoiseReduction',
         PrintConv => \%offOn,
     },
-    5 => {
-        Name => 'ColorMoireReduction',
+    0x05 => {
+        Name => 'ColorMoireReductionMode',
         PrintConv => {
             0 => 'Off',
             1 => 'Low',
@@ -388,15 +447,15 @@ my %unsharpColor = (
             3 => 'High',
         },
     },
-    9 => {
+    0x09 => {
         Name => 'NoiseReductionIntensity',
         Format => 'int32u',
     },
-    13 => {
+    0x0d => {
         Name => 'NoiseReductionSharpness',
         Format => 'int32u',
     },
-    17 => {
+    0x11 => {
         Name => 'NoiseReductionMethod',
         Format => 'int16u',
         PrintConv => {
@@ -404,6 +463,22 @@ my %unsharpColor = (
             1 => 'Better Quality',
         },
     },
+    0x15 => {
+        Name => 'ColorMoireReduction',
+        PrintConv => \%offOn,
+    },
+    0x17 => {
+        Name => 'NoiseReduction',
+        PrintConv => \%offOn,
+    },
+    0x18 => {
+        Name => 'ColorNoiseReductionIntensity',
+        Format => 'int32u',
+    },
+    0x1c => {
+        Name => 'ColorNoiseReductionSharpness',
+        Format => 'int32u',
+    },
 );
 
 %Image::ExifTool::NikonCapture::CropData = (
@@ -439,25 +514,25 @@ my %unsharpColor = (
         ValueConvInv => '$val * 2',
     },
     0x8e => {
-        Name => 'OutputWidthInches',
+        Name => 'CropOutputWidthInches',
         Format => 'double',
     },
     0x96 => {
-        Name => 'OutputHeightInches',
+        Name => 'CropOutputHeightInches',
         Format => 'double',
     },
     0x9e => {
-        Name => 'ScaledResolution',
+        Name => 'CropScaledResolution',
         Format => 'double',
     },
     0xae => {
-        Name => 'SourceResolution',
+        Name => 'CropSourceResolution',
         Format => 'double',
         ValueConv => '$val / 2',
         ValueConvInv => '$val * 2',
     },
     0xb6 => {
-        Name => 'OutputResolution',
+        Name => 'CropOutputResolution',
         Format => 'double',
     },
     0xbe => {
@@ -465,19 +540,69 @@ my %unsharpColor = (
         Format => 'double',
     },
     0xc6 => {
-        Name => 'OutputWidth',
+        Name => 'CropOutputWidth',
         Format => 'double',
     },
     0xce => {
-        Name => 'OutputHeight',
+        Name => 'CropOutputHeight',
         Format => 'double',
     },
     0xd6 => {
-        Name => 'OutputPixels',
+        Name => 'CropOutputPixels',
         Format => 'double',
     },
 );
 
+%Image::ExifTool::NikonCapture::PictureCtrl = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'int8u',
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0x00 => {
+        Name => 'PictureControlActive',
+        PrintConv => \%offOn,
+    },
+    0x13 => {
+        Name => 'PictureControlMode',
+        Format => 'string[16]',
+    },
+    # 0x29 changes with Hue and Sharpening
+    0x2a => {
+        Name => 'QuickAdjust',
+        ValueConv => '$val - 128',
+        ValueConvInv => '$val + 128',
+    },
+    0x2b => {
+        Name => 'SharpeningAdj',
+        ValueConv => '$val ? $val - 128 : "Auto"',
+        ValueConvInv => '$val=~/\d/ ? $val + 128 : 0',
+    },
+    0x2c => {
+        Name => 'ContrastAdj',
+        ValueConv => '$val ? $val - 128 : "Auto"',
+        ValueConvInv => '$val=~/\d/ ? $val + 128 : 0',
+    },
+    0x2d => {
+        Name => 'BrightnessAdj',
+        ValueConv => '$val ? $val - 128 : "Auto"', # no "Auto" mode (yet) for this setting
+        ValueConvInv => '$val=~/\d/ ? $val + 128 : 0',
+    },
+    0x2e => {
+        Name => 'SaturationAdj',
+        ValueConv => '$val ? $val - 128 : "Auto"',
+        ValueConvInv => '$val=~/\d/ ? $val + 128 : 0',
+    },
+    0x2f => {
+        Name => 'HueAdj',
+        ValueConv => '$val - 128',
+        ValueConvInv => '$val + 128',
+    },
+    # 0x37 changed from 0 to 2 when Picture Control is enabled (and no active DLighting)
+);
+
 %Image::ExifTool::NikonCapture::RedEyeData = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
@@ -496,6 +621,56 @@ my %unsharpColor = (
     },
 );
 
+%Image::ExifTool::NikonCapture::Exposure = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'int8u',
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0x00 => {
+        Name => 'ExposureAdj',
+        Format => 'int16s',
+        ValueConv => '$val / 100',
+        ValueConvInv => '$val * 100',
+    },
+    0x12 => {
+        Name => 'ExposureAdj2',
+        Format => 'double',
+        PrintConv => 'sprintf("%.4f", $val)',
+        PrintConvInvn => '$val',
+    },
+    0x24 => {
+        Name => 'ActiveD-Lighting',
+        PrintConv => \%offOn,
+    },
+    0x25 => {
+        Name => 'ActiveD-LightingMode',
+        PrintConv => {
+            0 => 'Unchanged',
+            1 => 'Off',
+            2 => 'Low',
+            3 => 'Normal',
+            4 => 'High',
+            6 => 'Extra High',
+        },
+    },
+);
+
+%Image::ExifTool::NikonCapture::HighlightData = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'int8s',
+    FIRST_ENTRY => 0,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0 => 'ShadowProtection',
+    1 => 'SaturationAdj',
+    6 => 'HighlightProtection',
+);
+
 #------------------------------------------------------------------------------
 # write Nikon Capture data (ref 1)
 # Inputs: 0) ExifTool object reference, 1) reference to directory information
@@ -513,7 +688,6 @@ sub WriteNikonCapture($$$)
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart};
     my $dirLen = $$dirInfo{DirLen};
-    my $dirEnd = $dirStart + $dirLen;
     if ($dirLen < 22) {
         $exifTool->Warn('Short Nikon Capture Data',1);
         return undef;
@@ -521,14 +695,24 @@ sub WriteNikonCapture($$$)
     # make sure the capture data is properly contained
     SetByteOrder('II');
     my $tagID = Get32u($dataPt, $dirStart);
+    # sometimes size includes 18 header bytes, and other times it doesn't (ie. ViewNX 2.1.1)
     my $size = Get32u($dataPt, $dirStart + 18);
-    unless ($tagID == 0x7a86a940 and $size + 18 == $dirLen) {
+    my $pad = $dirLen - $size - 18; 
+    unless ($tagID == 0x7a86a940 and ($pad >= 0 or $pad == -18)) {
         $exifTool->Warn('Unrecognized Nikon Capture Data header');
         return undef;
     }
+    # determine if there is any data after this block
+    if ($pad > 0) {
+        $pad = substr($$dataPt, $dirStart + 18 + $size, $pad);
+        $dirLen = $size + 18;
+    } else {
+        $pad = '';
+    }
     my $outBuff = '';
     my $pos;
     my $newTags = $exifTool->GetNewTagInfoHash($tagTablePtr);
+    my $dirEnd = $dirStart + $dirLen;
 
     # loop through all entries in the Nikon Capture data
     for ($pos=$dirStart+22; $pos+22<$dirEnd; $pos+=22+$size) {
@@ -557,8 +741,8 @@ sub WriteNikonCapture($$$)
                 # get new value for this tag if we are writing it
                 my $format = $$tagInfo{Format} || $$tagInfo{Writable};
                 my $oldVal = ReadValue($dataPt,$pos+22,$format,1,$size);
-                my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-                if (Image::ExifTool::IsOverwriting($newValueHash, $oldVal)) {
+                my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+                if (Image::ExifTool::IsOverwriting($nvHash, $oldVal)) {
                     my $val = $exifTool->GetNewValues($tagInfo);
                     $newVal = WriteValue($val, $$tagInfo{Writable}) if defined $val;
                     if (defined $newVal and length $newVal) {
@@ -582,19 +766,61 @@ sub WriteNikonCapture($$$)
         $outBuff .= substr($$dataPt, $pos, 22 + $size);
     }
     unless ($pos == $dirEnd) {
-        $exifTool->Warn('Nikon Capture Data improperly terminated',1);
-        return undef;
+        if ($pos == $dirEnd - 4) {
+            # it seems that sometimes (NX2) the main block size is wrong by 4 bytes
+            # (did they forget to include the size word?)
+            $outBuff .= substr($$dataPt, $pos, 4);
+        } else {
+            $exifTool->Warn('Nikon Capture Data improperly terminated',1);
+            return undef;
+        }
     }
     # add the header and return the new directory
     return substr($$dataPt, $dirStart, 18) .
            Set32u(length($outBuff) + 4) .
-           $outBuff;
+           $outBuff . $pad;
 }
 
 #------------------------------------------------------------------------------
 # process Nikon Capture data (ref 1)
-# Inputs: 0) ExifTool object reference, 1) reference to directory information
-#         2) pointer to tag table
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessNikonCaptureEditVersions($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart};
+    my $dirLen = $$dirInfo{DirLen};
+    my $dirEnd = $dirStart + $dirLen;
+    my $verbose = $exifTool->Options('Verbose');
+    SetByteOrder('II');
+    return 0 unless $dirLen > 4;
+    my $num = Get32u($dataPt, $dirStart);
+    my $pos = $dirStart + 4;
+    $verbose and $exifTool->VerboseDir('NikonCaptureEditVersions', $num);
+    while ($num) {
+        last if $pos + 4 > $dirEnd;
+        my $len = Get32u($dataPt, $pos);
+        last if $pos + $len + 4 > $dirEnd;
+        my %dirInfo = (
+            DirName  => 'NikonCapture',
+            Parent   => 'NikonCaptureEditVersions',
+            DataPt   => $dataPt,
+            DirStart => $pos + 4,
+            DirLen   => $len,
+        );
+        $$exifTool{DOC_NUM} = ++$$exifTool{DOC_COUNT};
+        $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        --$num;
+        $pos += $len + 4;
+    }
+    delete $$exifTool{DOC_NUM};
+    return 1;
+}
+
+#------------------------------------------------------------------------------
+# process Nikon Capture data (ref 1)
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessNikonCapture($$$)
 {
@@ -623,7 +849,13 @@ sub ProcessNikonCapture($$$)
                 $format = 'int' . ($size * 8) . 'u';
             }
             if ($format) {
-                $value = ReadValue($dataPt,$pos,$format,1,$size);
+                my $count = 1;
+                if ($format eq 'string' or $format eq 'undef') {
+                    # patch Nikon bug in size of some values (HistogramXML)
+                    $size += $$tagInfo{AdjustSize} if $tagInfo and $$tagInfo{AdjustSize};
+                    $count = $size;
+                }
+                $value = ReadValue($dataPt,$pos,$format,$count,$size);
             } elsif ($size == 1) {
                 $value = substr($$dataPt, $pos, $size);
             }
@@ -640,6 +872,8 @@ sub ProcessNikonCapture($$$)
 
 1;  # end
 
+__END__
+
 =head1 NAME
 
 Image::ExifTool::NikonCapture - Read/write Nikon Capture information
@@ -655,7 +889,7 @@ the maker notes of NEF images.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
