@@ -14,7 +14,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.11';
+$VERSION = '1.14';
 
 my %offOn = (
     0 => 'Off',
@@ -121,6 +121,7 @@ my %offOn = (
         Writable => 'int16u',
         PrintConv => \%offOn,
     },
+    # 0x0215 - Flash?
     0x0216 => {
         Name => 'VoiceMemo',
         Writable => 'int16u',
@@ -175,10 +176,16 @@ my %offOn = (
             6 => 'Lamp', #PH
         },
     },
-    0x0223 => {
-        Name => 'ManualFocusDistance',
-        Writable => 'rational64u',
-    },
+    0x0223 => [
+        {
+            Name => 'ManualFocusDistance',
+            Condition => '$format eq "rational64u"',
+            Writable => 'rational64u',
+        }, { #PH
+            Name => 'FaceInfo',
+            SubDirectory => { TagTable => 'Image::ExifTool::Sanyo::FaceInfo' },
+        },
+    ],
     0x0224 => {
         Name => 'SequenceShotInterval',
         Writable => 'int16u',
@@ -214,6 +221,26 @@ my %offOn = (
     },
 );
 
+# face detection information (ref PH)
+%Image::ExifTool::Sanyo::FaceInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    WRITABLE => 1,
+    FORMAT => 'int32u',
+    FIRST_ENTRY => 0,
+    0 => 'FacesDetected',
+    4 => {
+        Name => 'FacePosition',
+        Format => 'int32u[4]',
+        Notes => q{
+            left, top, right and bottom coordinates of detected face in an unrotated
+            640-pixel-wide image, with increasing Y downwards
+        },
+    },
+);
+
 # tags in Sanyo MOV videos (PH - observations from an E6 sample)
 # (similar information in Kodak,Minolta,Nikon,Olympus,Pentax and Sanyo videos)
 %Image::ExifTool::Sanyo::MOV = (
@@ -246,7 +273,7 @@ my %offOn = (
         Name => 'ExposureCompensation',
         Format => 'int32s',
         ValueConv => '$val / 10',
-        PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
+        PrintConv => 'Image::ExifTool::Exif::PrintFraction($val)',
     },
     0x44 => {
         Name => 'WhiteBalance',
@@ -269,6 +296,7 @@ my %offOn = (
 );
 
 # tags in Sanyo MP4 videos (PH - from C4, C5 and HD1A samples)
+# --> very similar to Samsung MP4 information
 # (there is still a lot more information here that could be decoded!)
 %Image::ExifTool::Sanyo::MP4 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -289,10 +317,14 @@ my %offOn = (
     # some of the shutter speeds should be around 1/500 or so)
     0x32 => {
         Name => 'FNumber',
-        Format => 'int32u',
-        ValueConv => '$val / 10',
+        Format => 'rational64u',
         PrintConv => 'sprintf("%.1f",$val)',
     },
+    0x3a => { # (NC)
+        Name => 'ExposureCompensation',
+        Format => 'rational64s',
+        PrintConv => '$val ? sprintf("%+.1f", $val) : 0',
+    },
     0x6a => {
         Name => 'ISO',
         Format => 'int32u',
@@ -381,7 +413,7 @@ Sanyo maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
