@@ -13,25 +13,27 @@
 #               6) Andrey Tverdokhleb private communication
 #               7) Rudiger Lange private communication (A700)
 #               8) Igal Milchtaich private communication
-#               9) Michael Reit private communication (DSC-TX7)
+#               9) Michael Reitinger private communication (DSC-TX7)
+#               10) http://www.klingebiel.com/tempest/hd/pmp.html
+#               11 Mike Battilana private communication
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Sony;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION %sonyLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Minolta;
 
-$VERSION = '1.40';
+$VERSION = '1.51';
 
 sub ProcessSRF($$$);
 sub ProcessSR2($$$);
 sub WriteSR2($$$);
 
-my %sonyLensTypes;  # filled in based on Minolta LensType's
+# (%sonyLensTypes is filled in based on Minolta LensType's)
 
 # ExposureProgram values (ref PH, mainly decoded from A200)
 my %sonyExposureProgram = (
@@ -51,6 +53,14 @@ my %sonyExposureProgram = (
     35 => 'Auto No Flash', # (A330)
 );
 
+my %binaryDataAttrs = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+);
+
 %Image::ExifTool::Sony::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
@@ -140,6 +150,29 @@ my %sonyExposureProgram = (
             TagTable => 'Image::ExifTool::PrintIM::Main',
         },
     },
+    # the next 3 tags have a different meaning for some models (with format int32u)
+    0x1000 => { #9 (F88, multi burst mode only)
+        Name => 'MultiBurstMode',
+        Condition => '$format eq "undef"',
+        Notes => 'MultiBurst tags valid only for models with this feature, like the F88',
+        Writable => 'undef',
+        Format => 'int8u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    0x1001 => { #9 (F88, multi burst mode only)
+        Name => 'MultiBurstImageWidth',
+        Condition => '$format eq "int16u"',
+        Writable => 'int16u',
+    },
+    0x1002 => { #9 (F88, multi burst mode only)
+        Name => 'MultiBurstImageHeight',
+        Condition => '$format eq "int16u"',
+        Writable => 'int16u',
+    },
+    0x1003 => { #9 (TX7, panorama mode only)
+        Name => 'Panorama',
+        SubDirectory => { TagTable => 'Image::ExifTool::Sony::Panorama' },
+    },
     0x2001 => { #PH (JPEG images from all DSLR's except the A100)
         Name => 'PreviewImage',
         Writable => 'undef',
@@ -163,13 +196,82 @@ my %sonyExposureProgram = (
             return Set32u(length $val) . $size . ("\0" x 8) . $size . ("\0" x 4) . $val;
         },
     },
+    # 0x2002 - probably Sharpness (PH guess)
+    0x2004 => { #PH (NEX-5)
+        Name => 'Contrast',
+        Writable => 'int32s',
+        PrintConv => '$val > 0 ? "+$val" : $val',
+        PrintConvInv => '$val',
+    },
+    0x2005 => { #PH (NEX-5)
+        Name => 'Saturation',
+        Writable => 'int32s',
+        PrintConv => '$val > 0 ? "+$val" : $val',
+        PrintConvInv => '$val',
+    },
+    0x2006 => { #PH
+        Name => 'Sharpness',
+        Writable => 'int32s',
+        PrintConv => '$val > 0 ? "+$val" : $val',
+        PrintConvInv => '$val',
+    },
+    0x2007 => { #PH
+        Name => 'Brightness',
+        Writable => 'int32s',
+        PrintConv => '$val > 0 ? "+$val" : $val',
+        PrintConvInv => '$val',
+    },
+    0x2008 => { #PH
+        Name => 'LongExposureNoiseReduction',
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On',
+            0xffff0000 => 'Off 2',
+            0xffff0001 => 'On 2',
+            0xffffffff => 'n/a',
+        },
+    },
+    0x2009 => { #PH
+        Name => 'HighISONoiseReduction',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Low',
+            2 => 'Normal',
+            3 => 'High',
+            256 => 'Auto',
+            65535 => 'n/a',
+        },
+    },
     0x200a => { #PH (A550)
-        Name => 'AutoHDR',
+        Name => 'HDR',
         Writable => 'int32u',
         PrintHex => 1,
         PrintConv => {
             0x0 => 'Off',
-            0x10001 => 'On',
+            0x10001 => 'Auto',
+            0x10010 => '1.0 EV', # (NEX_5)
+            0x10011 => '1.5 EV',
+            0x10012 => '2.0 EV',
+            0x10013 => '2.5 EV',
+            0x10014 => '3.0 EV',
+            0x10015 => '3.5 EV',
+            0x10016 => '4.0 EV',
+            0x10017 => '4.5 EV',
+            0x10018 => '5.0 EV',
+            0x10019 => '5.5 EV',
+            0x1001a => '6.0 EV', # (SLT-A55V)
+        },
+    },
+    0x200b => { #PH
+        Name => 'MultiFrameNoiseReduction',
+        Writable => 'int32u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On',
+            255 => 'n/a',
         },
     },
     0x3000 => {
@@ -194,6 +296,7 @@ my %sonyExposureProgram = (
             '2 0 0 0' => 'ARW 1.0',
             '3 0 0 0' => 'ARW 2.0',
             '3 1 0 0' => 'ARW 2.1',
+            '3 2 0 0' => 'ARW 2.2', # (NEX-5)
             # what about cRAW images?
         },
     },
@@ -210,12 +313,20 @@ my %sonyExposureProgram = (
             259 => 'DSLR-A200',
             260 => 'DSLR-A350',
             261 => 'DSLR-A300',
-            263 => 'DSLR-A380',
+            263 => 'DSLR-A380/A390', #PH (A390)
             264 => 'DSLR-A330',
             265 => 'DSLR-A230',
+            266 => 'DSLR-A290', #PH
             269 => 'DSLR-A850',
             273 => 'DSLR-A550',
             274 => 'DSLR-A500', #PH
+            275 => 'DSLR-A450', # (http://dev.exiv2.org/issues/show/0000611)
+            278 => 'NEX-5', #PH
+            279 => 'NEX-3', #PH
+            280 => 'SLT-A33', #PH
+            281 => 'SLT-A55V', #PH
+            282 => 'DSLR-A560', #PH
+            283 => 'DSLR-A580', # (http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,2881.0.html)
         },
     },
     0xb020 => { #2
@@ -232,7 +343,7 @@ my %sonyExposureProgram = (
     0xb022 => { #7
         Name => 'ColorCompensationFilter',
         Format => 'int32s',
-        Writable => 'int32u',
+        Writable => 'int32u', # (written incorrectly as unsigned by Sony)
         Notes => 'negative is green, positive is magenta',
     },
     0xb023 => { #PH (A100) - (set by mode dial)
@@ -256,12 +367,17 @@ my %sonyExposureProgram = (
             0 => 'Off',
             1 => 'Standard',
             2 => 'Advanced Auto',
-            3 => 'Auto', #PH (A550)
+            3 => 'Auto', # (A550)
             8 => 'Advanced Lv1', #JD
             9 => 'Advanced Lv2', #JD
             10 => 'Advanced Lv3', #JD
             11 => 'Advanced Lv4', #JD
             12 => 'Advanced Lv5', #JD
+            16 => 'Lv1', # (NEX_5)
+            17 => 'Lv2',
+            18 => 'Lv3',
+            19 => 'Lv4',
+            20 => 'Lv5',
         },
     },
     0xb026 => { #PH (A100)
@@ -272,12 +388,7 @@ my %sonyExposureProgram = (
     0xb027 => { #2
         Name => 'LensType',
         Writable => 'int32u',
-        Notes => q{
-            decimal values differentiate lenses which would otherwise have the same
-            LensType, and are used by the Composite LensID tag when attempting to
-            identify the specific lens model.  "New" or "II" appear in brackets if the
-            original version of the lens has the same LensType
-        },
+        SeparateTable => 1,
         PrintConv => \%sonyLensTypes,
     },
     0xb028 => { #2
@@ -319,11 +430,18 @@ my %sonyExposureProgram = (
     0xb040 => { #2
         Name => 'Macro',
         Writable => 'int16u',
-        PrintConv => { 0 => 'Off', 1 => 'On' },
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On',
+            2 => 'Close Focus', #9
+            65535 => 'n/a', #PH (A100)
+        },
     },
     0xb041 => { #2
         Name => 'ExposureMode',
         Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
         PrintConv => {
             0 => 'Auto',
             1 => 'Portrait', #PH (HX1)
@@ -338,8 +456,10 @@ my %sonyExposureProgram = (
             11 => 'Twilight Portrait', #9
             12 => 'Soft Snap', #9
             13 => 'Fireworks', #9
-            18 => 'High Sensitivity', #9
+            14 => 'Smile Shutter', #9 (T200)
             15 => 'Manual',
+            18 => 'High Sensitivity', #9
+            20 => 'Advanced Sports Shooting', #9
             29 => 'Underwater', #9
             33 => 'Gourmet', #9
             34 => 'Panorama', #PH (HX1)
@@ -347,20 +467,94 @@ my %sonyExposureProgram = (
             36 => 'Anti Motion Blur', #PH (TX1)
             37 => 'Pet', #9
             38 => 'Backlight Correction HDR', #9
+            65535 => 'n/a', #PH (A100)
+        },
+    },
+    0xb042 => { #9
+        Name => 'FocusMode',
+        Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            1 => 'AF-S', # (called Single-AF by Sony)
+            2 => 'AF-C', # (called Monitor-AF by Sony)
+            4 => 'Permanent-AF', # (TX7)
+            65535 => 'n/a', #PH (A100)
+        },
+    },
+    0xb043 => { #9
+        Name => 'AFMode',
+        Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Default', # (takes this value after camera reset, but can't be set back once changed)
+            1 => 'Multi AF',
+            2 => 'Center AF',
+            3 => 'Spot AF',
+            4 => 'Flexible Spot AF', # (T200)
+            6 => 'Touch AF',
+            14 => 'Manual Focus', # (T200)
+            15 => 'Face Detected', # (not set when in face detect mode and no faces detected)
+            65535 => 'n/a', #PH (A100)
+        },
+    },
+    0xb044 => { #9
+        Name => 'AFIlluminator',
+        Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Auto',
+            65535 => 'n/a', #PH (A100)
         },
     },
-    # 0xb043 - some sort of mode: values - 0,1,2,3,4,15(face detect?),65535
     0xb047 => { #2
         Name => 'Quality',
         Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
         PrintConv => {
             0 => 'Normal',
             1 => 'Fine',
+            65535 => 'n/a', #PH (A100)
+        },
+    },
+    0xb048 => { #9
+        Name => 'FlashLevel',
+        Writable => 'int16s',
+        RawConv => '$val == -1 ? undef : $val',
+        PrintConv => {
+            -32768 => 'Low',
+            -1 => 'n/a', #PH (A100)
+            0 => 'Normal',
+            32767 => 'High',
+        },
+    },
+    0xb049 => { #9
+        Name => 'ReleaseMode',
+        Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Normal',
+            2 => 'Burst',
+            5 => 'Exposure Bracketing',
+            6 => 'White Balance Bracketing', # (HX5)
+            65535 => 'n/a', #PH (A100)
+        },
+    },
+    0xb04a => { #9
+        Name => 'SequenceNumber',
+        Notes => 'shot number in continuous burst',
+        Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Single',
+            65535 => 'n/a', #PH (A100)
+            OTHER => sub { shift }, # pass all other numbers straight through
         },
     },
     0xb04b => { #2/PH
         Name => 'Anti-Blur',
         Writable => 'int16u',
+        RawConv => '$val == 65535 ? undef : $val',
         PrintConv => {
             0 => 'Off',
             1 => 'On (Continuous)', #PH (NC)
@@ -371,7 +565,12 @@ my %sonyExposureProgram = (
     0xb04e => { #2
         Name => 'LongExposureNoiseReduction',
         Writable => 'int16u',
-        PrintConv => { 0 => 'Off', 1 => 'On' },
+        RawConv => '$val == 65535 ? undef : $val',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On',
+            65535 => 'n/a', #PH (A100)
+        },
     },
     0xb04f => { #PH (TX1)
         Name => 'DynamicRangeOptimizer',
@@ -399,19 +598,52 @@ my %sonyExposureProgram = (
             0 => 'Auto',
             4 => 'Manual',
             5 => 'Daylight',
+            6 => 'Cloudy', #9
+            7 => 'White Flourescent', #9      (Sony "Fluorescent 1 (White)")
+            8 => 'Cool White Flourescent', #9 (Sony "Fluorescent 2 (Natural White)")
+            9 => 'Day White Flourescent', #9  (Sony "Fluorescent 3 (Day White)")
             14 => 'Incandescent',
+            15 => 'Flash', #9
+            17 => 'Underwater 1 (Blue Water)', #9
+            18 => 'Underwater 2 (Green Water)', #9
         },
     },
 );
 
+# "SEMC MS" maker notes
+%Image::ExifTool::Sony::Ericsson = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    NOTES => 'Maker notes found in images from some Sony Ericsson phones.',
+    0x2000 => {
+        Name => 'MakerNoteVersion',
+        Writable => 'undef',
+        Count => 4,
+    },
+    0x201 => {
+        Name => 'PreviewImageStart',
+        IsOffset => 1,
+        MakerPreview => 1, # force preview inside maker notes
+        OffsetPair => 0x202,
+        DataTag => 'PreviewImage',
+        Writable => 'int32u',
+        Protected => 2,
+        Notes => 'a small 320x200 preview image',
+    },
+    0x202 => {
+        Name => 'PreviewImageLength',
+        OffsetPair => 0x201,
+        DataTag => 'PreviewImage',
+        Writable => 'int32u',
+        Protected => 2,
+    },
+);
+
 # Camera settings (ref PH) (decoded mainly from A200)
 %Image::ExifTool::Sony::CameraSettings = (
-    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
-    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    %binaryDataAttrs,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
-    WRITABLE => 1,
-    FIRST_ENTRY => 0,
     FORMAT => 'int16u',
     NOTES => q{
         Camera settings for the A200, A230, A300, A350, A700, A850 and A900.  Some
@@ -680,12 +912,8 @@ my %sonyExposureProgram = (
 
 # Camera settings (ref PH) (A330 and A380)
 %Image::ExifTool::Sony::CameraSettings2 = (
-    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
-    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    %binaryDataAttrs,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
-    WRITABLE => 1,
-    FIRST_ENTRY => 0,
     FORMAT => 'int16u',
     NOTES => 'Camera settings for the A330 and A380.',
     0x10 => { #7 (A700, not confirmed for other models)
@@ -821,25 +1049,26 @@ my %sonyExposureProgram = (
 
 # Camera settings for other models
 %Image::ExifTool::Sony::CameraSettingsUnknown = (
-    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
-    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    %binaryDataAttrs,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
-    WRITABLE => 1,
-    FIRST_ENTRY => 0,
     FORMAT => 'int16u',
 );
 
 # shot information (ref PH)
 %Image::ExifTool::Sony::ShotInfo = (
-    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
-    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    %binaryDataAttrs,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    WRITABLE => 1,
-    FIRST_ENTRY => 0,
-    # 0 - byte order 'II'
-    6 => {
+    DATAMEMBER => [ 0x02, 0x30, 0x32 ],
+    IS_SUBDIR => [ 0x48, 0x5e ],
+    # 0x00 - byte order 'II'
+    0x02 => {
+        Name => 'FaceInfoOffset',
+        Format => 'int16u',
+        DataMember => 'FaceInfoOffset',
+        Writable => 0,
+        RawConv => '$$self{FaceInfoOffset} = $val',
+    },
+    0x06 => {
         Name => 'SonyDateTime',
         Format => 'string[20]',
         Groups => { 2 => 'Time' },
@@ -847,12 +1076,177 @@ my %sonyExposureProgram = (
         PrintConv => '$self->ConvertDateTime($val)',
         PrintConvInv => '$self->InverseDateTime($val,0)',
     },
-    #52 => {
-    #    # values: 'DC6303320222000' or 'DC5303320222000'
+    0x30 => { #Jeffrey Friedl
+        Name => 'FacesDetected',
+        DataMember => 'FacesDetected',
+        Format => 'int16u',
+        RawConv => '$$self{FacesDetected} = $val',
+    },
+    0x32 => {
+        Name => 'FaceInfoLength', # length of a single FaceInfo entry
+        DataMember => 'FaceInfoLength',
+        Format => 'int16u',
+        Writable => 0,
+        RawConv => '$$self{FaceInfoLength} = $val',
+    },
+    #0x34 => {
+    #    # values: 'DC5303320222000', 'DC6303320222000' or 'DC7303320222000'
     #    Name => 'UnknownString',
     #    Format => 'string[16]',
     #    Unknown => 1,
     #},
+    0x48 => { # (most models: DC5303320222000 and DC6303320222000)
+        Name => 'FaceInfo1',
+        Condition => q{
+            $$self{FacesDetected} and
+            $$self{FaceInfoOffset} == 0x48 and
+            $$self{FaceInfoLength} == 0x20
+        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Sony::FaceInfo1' },
+    },
+    0x5e => { # (HX7V: DC7303320222000)
+        Name => 'FaceInfo2',
+        Condition => q{
+            $$self{FacesDetected} and
+            $$self{FaceInfoOffset} == 0x5e and
+            $$self{FaceInfoLength} == 0x25
+        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Sony::FaceInfo2' },
+    },
+);
+
+%Image::ExifTool::Sony::FaceInfo1 = (
+    %binaryDataAttrs,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0x00 => {
+        Name => 'Face1Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 1 ? undef : $val',
+        Notes => q{
+            top, left, height and width of detected face.  Coordinates are relative to
+            the full-sized unrotated image, with increasing Y downwards
+        },
+    },
+    0x20 => {
+        Name => 'Face2Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 2 ? undef : $val',
+    },
+    0x40 => {
+        Name => 'Face3Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 3 ? undef : $val',
+    },
+    0x60 => {
+        Name => 'Face4Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 4 ? undef : $val',
+    },
+    0x80 => {
+        Name => 'Face5Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 5 ? undef : $val',
+    },
+    0xa0 => {
+        Name => 'Face6Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 6 ? undef : $val',
+    },
+    0xc0 => {
+        Name => 'Face7Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 7 ? undef : $val',
+    },
+    0xe0 => {
+        Name => 'Face8Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 8 ? undef : $val',
+    },
+);
+
+%Image::ExifTool::Sony::FaceInfo2 = (
+    %binaryDataAttrs,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0x00 => {
+        Name => 'Face1Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 1 ? undef : $val',
+        Notes => q{
+            top, left, height and width of detected face.  Coordinates are relative to
+            the full-sized unrotated image, with increasing Y downwards
+        },
+    },
+    0x25 => {
+        Name => 'Face2Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 2 ? undef : $val',
+    },
+    0x4a => {
+        Name => 'Face3Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 3 ? undef : $val',
+    },
+    0x6f => {
+        Name => 'Face4Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 4 ? undef : $val',
+    },
+    0x94 => {
+        Name => 'Face5Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 5 ? undef : $val',
+    },
+    0xb9 => {
+        Name => 'Face6Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 6 ? undef : $val',
+    },
+    0xde => {
+        Name => 'Face7Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 7 ? undef : $val',
+    },
+    0x103 => {
+        Name => 'Face8Position',
+        Format => 'int16u[4]',
+        RawConv => '$$self{FacesDetected} < 8 ? undef : $val',
+    },
+);
+
+# panorama info for cameras such as the HX1, HX5, TX7 (ref 9/PH)
+%Image::ExifTool::Sony::Panorama = (
+    %binaryDataAttrs,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    FORMAT => 'int32u',
+    NOTES => q{
+        Tags found only in panorama images from Sony cameras such as the HX1, HX5
+        and TX7.  The width/height values of these tags are not affected by camera
+        rotation -- the width is always the longer dimension.
+    },
+    # 0: 257
+    1 => 'PanoramaFullWidth', # (including black/grey borders)
+    2 => 'PanoramaFullHeight',
+    3 => {
+        Name => 'PanoramaDirection',
+        PrintConv => {
+            0 => 'Right to Left',
+            1 => 'Left to Right',
+        },
+    },
+    # crop area to remove black/grey borders from full image
+    4 => 'PanoramaCropLeft',
+    5 => 'PanoramaCropTop', #PH guess (NC)
+    6 => 'PanoramaCropRight',
+    7 => 'PanoramaCropBottom',
+    # 8: 1728 (HX1), 1824 (HX5/TX7) (value8/value9 = 16/9)
+    8 => 'PanoramaFrameWidth', #PH guess (NC)
+    # 9: 972 (HX1), 1026 (HX5/TX7)
+    9 => 'PanoramaFrameHeight', #PH guess (NC)
+    # 10: 3200-3800 (HX1), 4000-4900 (HX5/TX7)
+    10 => 'PanoramaSourceWidth', #PH guess (NC)
+    # 11: 800-1800 (larger for taller panoramas)
+    11 => 'PanoramaSourceHeight', #PH guess (NC)
+    # 12-15: 0
 );
 
 # tag table for SRF0 IFD (ref 1)
@@ -1002,16 +1396,118 @@ my %sonyExposureProgram = (
     },
 );
 
+# tags found in DSC-F1 PMP header (ref 10)
+%Image::ExifTool::Sony::PMP = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    FIRST_ENTRY => 0,
+    NOTES => q{
+        These tags are written in the proprietary-format header of PMP images from
+        the DSC-F1.
+    },
+    8 => { #PH
+        Name => 'JpgFromRawStart',
+        Format => 'int32u',
+        Notes => q{
+            OK, not really a RAW file, but this mechanism is used to allow extraction of
+            the JPEG image from a PMP file
+        },
+    },
+    12 => { Name => 'JpgFromRawLength',Format => 'int32u' },
+    22 => { Name => 'SonyImageWidth',  Format => 'int16u' },
+    24 => { Name => 'SonyImageHeight', Format => 'int16u' },
+    27 => {
+        Name => 'Orientation',
+        PrintConv => {
+            0 => 'Horizontal (normal)',
+            1 => 'Rotate 270 CW',#11
+            2 => 'Rotate 180',
+            3 => 'Rotate 90 CW',#11
+        },
+    },
+    29 => {
+        Name => 'ImageQuality',
+        PrintConv => {
+            8 => 'Snap Shot',
+            23 => 'Standard',
+            51 => 'Fine',
+        },
+    },
+    # 40 => ImageWidth again (int16u)
+    # 42 => ImageHeight again (int16u)
+    52 => { Name => 'Comment',         Format => 'string[19]' },
+    76 => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Format => 'int8u[6]',
+        Groups => { 2 => 'Time' },
+        ValueConv => q{
+            my @a = split ' ', $val;
+            $a[0] += $a[0] < 70 ? 2000 : 1900;
+            sprintf('%.4d:%.2d:%.2d %.2d:%.2d:%.2d', @a);
+        },
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    84 => {
+        Name => 'ModifyDate',
+        Format => 'int8u[6]',
+        Groups => { 2 => 'Time' },
+        ValueConv => q{
+            my @a = split ' ', $val;
+            $a[0] += $a[0] < 70 ? 2000 : 1900;
+            sprintf('%.4d:%.2d:%.2d %.2d:%.2d:%.2d', @a);
+        },
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    102 => {
+        Name => 'ExposureTime',
+        Format => 'int16s',
+        RawConv => '$val <= 0 ? undef : $val',
+        ValueConv => '2 ** (-$val / 100)',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+    },
+    106 => { # (NC -- not written by DSC-F1)
+        Name => 'FNumber',
+        Format => 'int16s',
+        RawConv => '$val <= 0 ? undef : $val',
+        ValueConv => '$val / 100', # (likely wrong)
+    },
+    108 => { # (NC -- not written by DSC-F1)
+        Name => 'ExposureCompensation',
+        Format => 'int16s',
+        RawConv => '($val == -1 or $val == -32768) ? undef : $val',
+        ValueConv => '$val / 100', # (probably wrong too)
+    },
+    112 => { # (NC -- not written by DSC-F1)
+        Name => 'FocalLength',
+        Format => 'int16s',
+        Groups => { 2 => 'Camera' },
+        RawConv => '$val <= 0 ? undef : $val',
+        ValueConv => '$val / 100',
+        PrintConv => 'sprintf("%.1f mm",$val)',
+    },
+    118 => {
+        Name => 'Flash',
+        Groups => { 2 => 'Camera' },
+        PrintConv => { 0 => 'No Flash', 1 => 'Fired' },
+    },
+);
+
 # fill in Sony LensType lookup based on Minolta values
 {
-    %sonyLensTypes = %Image::ExifTool::Minolta::minoltaLensTypes;
+    my $minoltaTypes = \%Image::ExifTool::Minolta::minoltaLensTypes;
+    %sonyLensTypes = %$minoltaTypes;
+    my $notes = $$minoltaTypes{Notes};
+    delete $$minoltaTypes{Notes};
     my $id;
-    foreach $id (sort { $a <=> $b } keys %Image::ExifTool::Minolta::minoltaLensTypes) {
+    foreach $id (sort { $a <=> $b } keys %$minoltaTypes) {
         # higher numbered lenses are missing last digit of ID for some Sony models
         next if $id < 10000;
         my $sid = int($id/10);
         my $i;
-        my $lens = $Image::ExifTool::Minolta::minoltaLensTypes{$id};
+        my $lens = $$minoltaTypes{$id};
         if ($sonyLensTypes{$sid}) {
             # put lens name with "or" first in list
             if ($lens =~ / or /) {
@@ -1027,6 +1523,34 @@ my %sonyExposureProgram = (
         }
         $sonyLensTypes{$sid} = $lens;
     }
+    $$minoltaTypes{Notes} = $sonyLensTypes{Notes} = $notes;
+}
+
+#------------------------------------------------------------------------------
+# Read Sony DSC-F1 PMP file
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref
+# Returns: 1 on success when reading, 0 if this isn't a valid PMP file
+sub ProcessPMP($$)
+{
+    my ($exifTool, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $buff;
+    $raf->Read($buff, 128) == 128 or return 0;
+    # validate header length (124 bytes)
+    $buff =~ /^.{8}\0{3}\x7c.{112}\xff\xd8\xff\xdb$/s or return 0;
+    $exifTool->SetFileType();
+    SetByteOrder('MM');
+    $exifTool->FoundTag(Make => 'Sony');
+    $exifTool->FoundTag(Model => 'DSC-F1');
+    # extract information from 124-byte header
+    my $tagTablePtr = GetTagTable('Image::ExifTool::Sony::PMP');
+    my %dirInfo = ( DataPt => \$buff, DirName => 'PMP' );
+    $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+    # process JPEG image
+    $raf->Seek(124, 0);
+    $$dirInfo{Base} = 124;
+    $exifTool->ProcessJPEG($dirInfo);
+    return 1;
 }
 
 #------------------------------------------------------------------------------
@@ -1162,7 +1686,6 @@ sub ProcessSRF($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
-    my $dirLen = $$dirInfo{DirLen};
     my $start = $$dirInfo{DirStart};
     my $verbose = $exifTool->Options('Verbose');
 
@@ -1307,7 +1830,6 @@ sub ProcessSR2($$$)
         $exifTool->Warn($err);
         return 0;
     }
-    my $dirLen = $$dirInfo{DirLen};
     my $verbose = $exifTool->Options('Verbose');
     my $result;
     if ($outfile) {
@@ -1447,7 +1969,7 @@ Minolta.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1460,13 +1982,15 @@ under the same terms as Perl itself.
 
 =item L<http://homepage3.nifty.com/kamisaka/makernote/makernote_sony.htm>
 
+=item L<http://www.klingebiel.com/tempest/hd/pmp.html>
+
 =back
 
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Thomas Bodenmann, Philippe Devaux, Jens Duttke, Marcus
-Holland-Moritz, Andrey Tverdokhleb, Rudiger Lange and Igal Milchtaich for
-help decoding some tags.
+Holland-Moritz, Andrey Tverdokhleb, Rudiger Lange, Igal Milchtaich and
+Michael Reitinger for help decoding some tags.
 
 =head1 SEE ALSO
 
