@@ -12,6 +12,7 @@
 #               May  18/05 - P. Harvey Tolerate round-off errors in floats.
 #               Feb. 02/08 - P. Harvey Allow different timezones in time values
 #               Sep. 16/08 - P. Harvey Improve timezone testing
+#               Jul. 14/10 - P. Harvey Added writeInfo()
 #------------------------------------------------------------------------------
 
 package t::TestLib;
@@ -22,15 +23,16 @@ require Exporter;
 use Image::ExifTool qw(ImageInfo);
 
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '1.15';
+$VERSION = '1.17';
 @ISA = qw(Exporter);
-@EXPORT = qw(check writeCheck testCompare binaryCompare testVerbose);
+@EXPORT = qw(check writeCheck writeInfo testCompare binaryCompare testVerbose);
 
 sub closeEnough($$);
 sub formatValue($);
+sub writeInfo($$;$$);
 
 #------------------------------------------------------------------------------
-# compare 2 binary files
+# Compare 2 binary files
 # Inputs: 0) file name 1, 1) file name 2
 # Returns: 1 if files are identical
 sub binaryCompare($$)
@@ -57,11 +59,11 @@ sub binaryCompare($$)
 
 #------------------------------------------------------------------------------
 # Compare 2 files and return true and erase the 2nd file if they are the same
-# Inputs: 0) file1, 1) file2
+# Inputs: 0) file1, 1) file2, 2) test number, 3) flag to not erase test file
 # Returns: true if files are the same
-sub testCompare($$$)
+sub testCompare($$$;$)
 {
-    my ($stdfile, $testfile, $testnum) = @_;
+    my ($stdfile, $testfile, $testnum, $keep) = @_;
     my $success = 0;
     my $linenum;
     
@@ -108,7 +110,7 @@ sub testCompare($$$)
     $/ = $oldSep;       # restore input line separator
     
     # erase .failed file if test was successful
-    $success and unlink $testfile;
+    $success and not $keep and unlink $testfile;
     
     return $success
 }
@@ -151,8 +153,8 @@ sub closeEnough($$)
         my $tok2 = $toks2[$i];
         last unless defined $tok1 and defined $tok2;
         next if $tok1 eq $tok2;
-        # can't compare any more if either line was truncated (ie. ends with '[...]')
-        return $lenChanged if $tok1 =~ /\Q[...]\E$/ or $tok2 =~ /\Q[...]\E$/;
+        # can't compare any more if either line was truncated (ie. ends with '[...]' or '[snip]')
+        return $lenChanged if $tok1 =~ /\[(\.{3}|snip)\]$/ or $tok2 =~ /\[(\.{3}|snip)\]$/;
         # account for different timezones
         if ($tok1 =~ /^(\d{2}:\d{2}:\d{2})(Z|[-+]\d{2}:\d{2})$/i) {
             my $time = $1;  # remove timezone
@@ -180,8 +182,17 @@ sub closeEnough($$)
             last unless $t1 == $t2;
         } else {
             # check to see if both tokens are floating point numbers (with decimal points!)
-            $tok1 =~ s/[^\d.]+$//; $tok2 =~ s/[^\d.]+$//;   # remove trailing units
-            $tok1 =~ s/^\d+:\d+://; $tok2 =~ s/^\d+:\d+://; # remove HH:MM:
+            if ($tok1 =~ s/([^\d.]+)$//) {  # remove trailing units
+                my $a = $1;
+                last unless $tok2 =~ s/\Q$a\E$//;
+            }
+            if ($tok1 =~ s/^(\d+:\d+:)//) { # remove leading HH:MM:
+                my $a = $1;
+                last unless $tok2 =~ s/^\Q$a//;
+            }
+            if ($tok1 =~ s/^'//) {          # remove leading quote
+                last unless $tok2 =~ s/^'//;
+            }
             last unless Image::ExifTool::IsFloat($tok1) and
                         Image::ExifTool::IsFloat($tok2) and
                         $tok1 =~ /\./ and $tok2 =~ /\./;
@@ -192,12 +203,11 @@ sub closeEnough($$)
                 my ($int1, $int2);
                 ($int1 = $tok1) =~ tr/0-9//dc;
                 ($int2 = $tok2) =~ tr/0-9//dc;
-                if (length($int1) != length($int2)) {
-                    if (length($int1) > length($int2)) {
-                        $int1 = substr($int1, 0, length($int2));
-                    } else {
-                        $int2 = substr($int2, 0, length($int1));
-                    }
+                my $dlen = length($int1) - length($int2);
+                if ($dlen > 0) {
+                    $int2 .= '0' x $dlen;
+                } elsif ($dlen < 0) {
+                    $int1 .= '0' x (-$dlen);
                 }
                 last if abs($int1-$int2) > 1.00001;
             }
@@ -227,14 +237,14 @@ sub formatValue($)
         foreach (@$val) {
             push @a, formatValue($_);
         }
-        $str = join(', ', @a);
+        $str = '[' . join(',', @a) . ']';
     } elsif (ref $val eq 'HASH') {
         my $key;
         foreach $key (sort keys %$val) {
             push @a, $key . '=' . formatValue($$val{$key});
         }
-        $str = '{ ' . join('; ',@a) . ' }';
-    } else {
+        $str = '{' . join(',', @a) . '}';
+    } elsif (defined $val) {
         # make sure there are no linefeeds in output
         ($str = $val) =~ tr/\x0a\x0d/;/;
         # translate unknown characters
@@ -242,6 +252,8 @@ sub formatValue($)
         $str =~ tr/\x01-\x1f\x7f/./;
         # remove NULL chars
         $str =~ s/\x00//g;
+    } else {
+        $str = '';
     }
     return $str;
 }
@@ -301,14 +313,15 @@ sub check($$$;$$$)
 }
 
 #------------------------------------------------------------------------------
-# test writing feature by writing specified information to JPEG file
+# Test writing feature by writing specified information to JPEG file
 # Inputs: 0) list reference to lists of SetNewValue arguments
 #         1) test name, 2) test number, 3) optional source file name,
 #         4) true to only check tags which were written (or list ref for tags to check)
+#         5) flag set if nothing is expected to change in the output file
 # Returns: 1 if check passed
-sub writeCheck($$$;$$)
+sub writeCheck($$$;$$$)
 {
-    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten) = @_;
+    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten, $same) = @_;
     $srcfile or $srcfile = "t/images/$testname.jpg";
     my ($ext) = ($srcfile =~ /\.(.+?)$/);
     my $testfile = "t/${testname}_${testnum}_failed.$ext";
@@ -323,17 +336,39 @@ sub writeCheck($$$;$$)
         push @tags, $$_[0] if $onlyWritten;
     }
     unlink $testfile;
-    $exifTool->WriteInfo($srcfile, $testfile);
-    my $info = $exifTool->GetInfo('Error');
-    foreach (keys %$info) { warn "$$info{$_}\n"; }
-    $info = $exifTool->ImageInfo($testfile,{Duplicates=>1,Unknown=>1},@tags);
+    my $ok = writeInfo($exifTool, $srcfile, $testfile, $same);
+    my $info = $exifTool->ImageInfo($testfile,{Duplicates=>1,Unknown=>1},@tags);
     my $rtnVal = check($exifTool, $info, $testname, $testnum);
-    $rtnVal and unlink $testfile;
-    return $rtnVal;
+    return 0 unless $ok and $rtnVal;
+    unlink $testfile;
+    return 1;
 }
 
 #------------------------------------------------------------------------------
-# test verbose output
+# Call Image::ExifTool::WriteInfo with error checking
+# Inputs: 0) ExifTool ref, 1) src file, 2) dst file, 3) true if nothing should change
+# Return: true on success
+sub writeInfo($$;$$)
+{
+    my ($exifTool, $src, $dst, $same) = @_;
+    # erase temporary file created by WriteInfo() if no destination file is given
+    # (may be left over from previous crashed tests)
+    unlink "${src}_exiftool_tmp" if not defined $dst and not ref $src;
+    my $result = $exifTool->WriteInfo($src, $dst);
+    my $err = '';
+    $err .= "  Error: WriteInfo() returned $result\n" if $result != ($same ? 2 : 1);
+    my $info = $exifTool->GetInfo('Warning', 'Error');
+    foreach (sort keys %$info) {
+        my $tag = Image::ExifTool::GetTagName($_);
+        $err .= "  $tag: $$info{$_}\n";
+    }
+    return 1 unless $err;
+    warn "\n$err";
+    return 0;
+}
+
+#------------------------------------------------------------------------------
+# Test verbose output
 # Inputs: 0) test name, 1) test number, 2) Input file, 3) verbose level
 # Returns: true if test passed
 sub testVerbose($$$$)
